"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.groupChangesets = exports.filterChangesets = exports.searchChangesets = exports.listChangesets = exports.FormatMode = exports.OutputMode = exports.GroupMode = exports.SearchMode = exports.getUnityChangeset = exports.UnityChangeset = void 0;
const unityChangeset_js_1 = require("./unityChangeset.js");
const unityGraphQL_js_1 = require("./unityGraphQL.js");
exports.UnityChangeset = unityChangeset_js_1.UnityChangeset;
/*
 * Get an Unity changeset from specific Unity version.
 * @param version The Unity version.
 * @returns An Unity changeset.
 */
async function getUnityChangeset(version) {
    const changesets = (await (0, unityGraphQL_js_1.getUnityReleases)(version, []))
        .filter((c) => c.version === version);
    if (0 < changesets.length) {
        return changesets[0];
    }
    throw Error(`The given version '${version}' was not found.`);
}
exports.getUnityChangeset = getUnityChangeset;
/*
 * Search mode.
 *
 * All: All changesets.
 * Default: Only non pre-release changesets.
 * PreRelease: Only pre-release (alpha/beta) changesets.
 */
var SearchMode;
(function (SearchMode) {
    SearchMode[SearchMode["All"] = 0] = "All";
    SearchMode[SearchMode["Default"] = 2] = "Default";
    SearchMode[SearchMode["PreRelease"] = 3] = "PreRelease";
})(SearchMode || (exports.SearchMode = SearchMode = {}));
/*
 * Group mode.
 *
 * All: All the changesets.
 * OldestPatch: Only the oldest patch changesets.
 * LatestPatch: Only the latest patch changesets.
 * LatestLifecycle: Only the latest lifecycle changesets.
 */
var GroupMode;
(function (GroupMode) {
    GroupMode["All"] = "all";
    GroupMode["OldestPatch"] = "oldest-patch";
    GroupMode["LatestPatch"] = "latest-patch";
    GroupMode["LatestLifecycle"] = "latest-lifecycle";
})(GroupMode || (exports.GroupMode = GroupMode = {}));
/*
 * Output mode.
 *
 * Changeset: The changeset.
 * VersionOnly: Only the version.
 * MinorVersionOnly: Only the minor version.
 */
var OutputMode;
(function (OutputMode) {
    OutputMode["Changeset"] = "changeset";
    OutputMode["VersionOnly"] = "version";
    OutputMode["MinorVersionOnly"] = "minor-version";
})(OutputMode || (exports.OutputMode = OutputMode = {}));
/*
 * Format mode.
 *
 * None: No format.
 * Json: JSON format.
 * PrettyJson: Pretty JSON format.
 */
var FormatMode;
(function (FormatMode) {
    FormatMode["None"] = "none";
    FormatMode["Json"] = "json";
    FormatMode["PrettyJson"] = "pretty-json";
})(FormatMode || (exports.FormatMode = FormatMode = {}));
function listChangesets(searchMode, filterOptions, groupMode, outputMode, formatMode) {
    return searchChangesets(searchMode)
        .then((results) => filterChangesets(results, filterOptions))
        .then((results) => results.sort((a, b) => b.versionNumber - a.versionNumber))
        .then((results) => groupChangesets(results, groupMode))
        .then((results) => {
        switch (outputMode) {
            case OutputMode.Changeset:
                return results;
            case OutputMode.VersionOnly:
                return results.map((c) => c.version);
            case OutputMode.MinorVersionOnly:
                return results.map((c) => c.minor);
            default:
                throw Error(`The given output mode '${outputMode}' was not supported`);
        }
    })
        .then((results) => {
        switch (formatMode) {
            case FormatMode.None:
                return results.join("\n");
            case FormatMode.Json:
                return JSON.stringify(results);
            case FormatMode.PrettyJson:
                return JSON.stringify(results, null, 2);
            default:
                throw Error(`The given format mode '${formatMode}' was not supported`);
        }
    });
}
exports.listChangesets = listChangesets;
function searchChangesets(searchMode) {
    switch (searchMode) {
        case SearchMode.All:
            return (0, unityGraphQL_js_1.getUnityReleases)(".", []);
        case SearchMode.Default:
            return (0, unityGraphQL_js_1.getUnityReleases)(".", [
                unityGraphQL_js_1.UnityReleaseStream.LTS,
                unityGraphQL_js_1.UnityReleaseStream.TECH,
            ]);
        case SearchMode.PreRelease:
            return (0, unityGraphQL_js_1.getUnityReleases)(".", [
                unityGraphQL_js_1.UnityReleaseStream.BETA,
                unityGraphQL_js_1.UnityReleaseStream.ALPHA,
            ]);
        default:
            throw Error(`The given search mode '${searchMode}' was not supported`);
    }
}
exports.searchChangesets = searchChangesets;
function filterChangesets(changesets, options) {
    if (!changesets || changesets.length == 0)
        return [];
    // Min version number
    const min = options.min
        ? exports.UnityChangeset.toNumber(options.min, false)
        : Number.MIN_VALUE;
    // Max version number
    const max = options.max
        ? exports.UnityChangeset.toNumber(options.max, true)
        : Number.MAX_VALUE;
    // Grep pattern
    const regex = options.grep ? new RegExp(options.grep, "i") : null;
    // Lifecycle filter
    const lc = options.allLifecycles
        ? null
        : Object.values(groupBy(changesets, (r) => r.minor))
            .map((g) => g[0]);
    return changesets
        .filter((c) => min <= c.versionNumber &&
        c.versionNumber <= max &&
        (!options.lts || c.lts) &&
        (!regex || regex.test(c.version)) &&
        (!lc || lc.some((l) => l.minor == c.minor && l.lifecycle == c.lifecycle)));
}
exports.filterChangesets = filterChangesets;
function groupChangesets(changesets, groupMode) {
    if (!changesets || changesets.length == 0)
        return [];
    switch (groupMode) {
        case GroupMode.All:
            return changesets;
        case GroupMode.LatestLifecycle:
            return Object.values(groupBy(changesets, (r) => r.minor))
                .map((g) => g.filter((v) => v.lifecycle == g[0].lifecycle))
                .flat();
        case GroupMode.LatestPatch:
            return Object.values(groupBy(changesets, (r) => r.minor))
                .map((g) => g[0]);
        case GroupMode.OldestPatch:
            return Object.values(groupBy(changesets, (r) => r.minor))
                .map((g) => g.filter((v) => v.lifecycle == g[0].lifecycle))
                .map((g) => g[g.length - 1]);
        default:
            throw Error(`The given group mode '${groupMode}' was not supported`);
    }
}
exports.groupChangesets = groupChangesets;
const groupBy = (arr, key) => arr.reduce((groups, item) => {
    (groups[key(item)] ||= []).push(item);
    return groups;
}, {});
